@@ -17,6 +17,8 @@
  * under the License.
  */
 
+pub(crate) mod util;
+
 use std::any::Any;
 use std::collections::HashMap;
 use std::fmt::Debug;
@@ -31,19 +33,48 @@ use datafusion::datasource::object_store::ObjectStoreUrl;
 use datafusion::datasource::physical_plan::parquet::ParquetExecBuilder;
 use datafusion::datasource::physical_plan::FileScanConfig;
 use datafusion::datasource::TableProvider;
+use datafusion::logical_expr::Operator;
 use datafusion::physical_plan::ExecutionPlan;
 use datafusion_common::config::TableParquetOptions;
 use datafusion_common::DFSchema;
 use datafusion_common::DataFusionError::Execution;
 use datafusion_common::Result;
-use datafusion_expr::{CreateExternalTable, Expr, TableType};
+use datafusion_expr::{CreateExternalTable, Expr, TableProviderFilterPushDown, TableType};
 use datafusion_physical_expr::create_physical_expr;
 
+use crate::util::expr::exprs_to_filters;
 use hudi_core::config::read::HudiReadConfig::InputPartitions;
-use hudi_core::config::utils::empty_options;
-use hudi_core::storage::utils::{get_scheme_authority, parse_uri};
+use hudi_core::config::util::empty_options;
+use hudi_core::storage::util::{get_scheme_authority, join_url_segments};
 use hudi_core::table::Table as HudiTable;
+use hudi_core::util::StrTupleRef;
 
+/// Create a `HudiDataSource`.
+/// Used for Datafusion to query Hudi tables
+///
+/// # Examples
+///
+/// ```rust
+/// use std::sync::Arc;
+///
+/// use datafusion::error::Result;
+/// use datafusion::prelude::{DataFrame, SessionContext};
+/// use hudi_datafusion::HudiDataSource;
+///
+/// // Initialize a new DataFusion session context
+/// let ctx = SessionContext::new();
+///
+/// // Create a new HudiDataSource with specific read options
+/// let hudi = HudiDataSource::new_with_options(
+///     "/tmp/trips_table",
+///     [("hoodie.read.input.partitions", 5)]).await?;
+///
+/// // Register the Hudi table with the session context
+/// ctx.register_table("trips_table", Arc::new(hudi))?;
+/// let df: DataFrame = ctx.sql("SELECT * from trips_table where city = 'san_francisco'").await?;
+/// df.show().await?;
+///
+/// ```
 #[derive(Clone, Debug)]
 pub struct HudiDataSource {
     table: Arc<HudiTable>,
@@ -72,6 +103,47 @@ impl HudiDataSource {
             .get_or_default(InputPartitions)
             .to::<usize>()
     }
+
+    /// Check if the given expression can be pushed down to the Hudi table.
+    ///
+    /// The expression can be pushed down if it is a binary expression with a supported operator and operands.
+    fn can_push_down(&self, expr: &Expr) -> bool {
+        match expr {
+            Expr::BinaryExpr(binary_expr) => {
+                let left = &binary_expr.left;
+                let op = &binary_expr.op;
+                let right = &binary_expr.right;
+                self.is_supported_operator(op)
+                    && self.is_supported_operand(left)
+                    && self.is_supported_operand(right)
+            }
+            Expr::Not(inner_expr) => {
+                // Recursively check if the inner expression can be pushed down
+                self.can_push_down(inner_expr)
+            }
+            _ => false,
+        }
+    }
+
+    fn is_supported_operator(&self, op: &Operator) -> bool {
+        matches!(
+            op,
+            Operator::Eq
+                | Operator::NotEq
+                | Operator::Gt
+                | Operator::Lt
+                | Operator::GtEq
+                | Operator::LtEq
+        )
+    }
+
+    fn is_supported_operand(&self, expr: &Expr) -> bool {
+        match expr {
+            Expr::Column(col) => self.schema().column_with_name(&col.name).is_some(),
+            Expr::Literal(_) => true,
+            _ => false,
+        }
+    }
 }
 
 #[async_trait]
@@ -103,31 +175,33 @@ impl TableProvider for HudiDataSource {
     ) -> Result<Arc<dyn ExecutionPlan>> {
         self.table.register_storage(state.runtime_env().clone());
 
+        // Convert Datafusion `Expr` to `Filter`
+        let pushdown_filters = exprs_to_filters(filters);
         let file_slices = self
             .table
-            // TODO: implement supports_filters_pushdown() to pass filters to Hudi table API
-            .get_file_slices_splits(self.get_input_partitions(), &[])
+            .get_file_slices_splits(self.get_input_partitions(), &pushdown_filters.as_strs())
             .await
             .map_err(|e| Execution(format!("Failed to get file slices from Hudi table: {}", e)))?;
+        let base_url = self.table.base_url();
         let mut parquet_file_groups: Vec<Vec<PartitionedFile>> = Vec::new();
         for file_slice_vec in file_slices {
-            let parquet_file_group_vec = file_slice_vec
-                .iter()
-                .map(|f| {
-                    let url = parse_uri(&f.base_file.info.uri).unwrap();
-                    let size = f.base_file.info.size as u64;
-                    PartitionedFile::new(url.path(), size)
-                })
-                .collect();
+            let mut parquet_file_group_vec = Vec::new();
+            for f in file_slice_vec {
+                let relative_path = f.base_file_relative_path().map_err(|e| {
+                    Execution(format!(
+                        "Failed to get base file relative path for {f:?} due to {e:?}"
+                    ))
+                })?;
+                let url = join_url_segments(&base_url, &[relative_path.as_str()])
+                    .map_err(|e| Execution(format!("Failed to join URL segments: {e:?}")))?;
+                let size = f.base_file.file_metadata.as_ref().map_or(0, |m| m.size);
+                let partitioned_file = PartitionedFile::new(url.path(), size as u64);
+                parquet_file_group_vec.push(partitioned_file);
+            }
             parquet_file_groups.push(parquet_file_group_vec)
         }
 
-        let base_url = self.table.base_url().map_err(|e| {
-            Execution(format!(
-                "Failed to get base path config from Hudi table: {}",
-                e
-            ))
-        })?;
+        let base_url = self.table.base_url();
         let url = ObjectStoreUrl::parse(get_scheme_authority(&base_url))?;
         let fsc = FileScanConfig::new(url, self.schema())
             .with_file_groups(parquet_file_groups)
@@ -150,8 +224,59 @@ impl TableProvider for HudiDataSource {
 
         Ok(exec_builder.build_arc())
     }
+
+    fn supports_filters_pushdown(
+        &self,
+        filters: &[&Expr],
+    ) -> Result<Vec<TableProviderFilterPushDown>> {
+        filters
+            .iter()
+            .map(|expr| {
+                if self.can_push_down(expr) {
+                    Ok(TableProviderFilterPushDown::Inexact)
+                } else {
+                    Ok(TableProviderFilterPushDown::Unsupported)
+                }
+            })
+            .collect()
+    }
 }
 
+/// `HudiTableFactory` is responsible for creating and configuring Hudi tables.
+///
+/// This factory handles the initialization of Hudi tables by creating configuration
+/// options from both session state and table creation commands.
+///
+/// # Examples
+///
+/// Creating a new `HudiTableFactory` instance:
+///
+/// ```rust
+/// use datafusion::prelude::SessionContext;
+/// use hudi_datafusion::HudiTableFactory;
+///
+/// // Initialize a new HudiTableFactory
+/// let factory = HudiTableFactory::new();
+///     
+/// // The factory can be used to create Hudi tables
+/// let table = factory.create_table(...)?;
+///
+///
+/// // Using `CREATE EXTERNAL TABLE` to register Hudi table:
+/// let test_table =  factory.create_table(...)?; // Table with path + url
+/// let ctx = SessionContext::new();
+///
+/// // Register table in session using `CREATE EXTERNAL TABLE` command
+/// let create_table_sql = format!(
+///     "CREATE EXTERNAL TABLE {} STORED AS HUDI LOCATION '{}' {}",
+///     test_table.as_ref(),
+///     test_table.path(),
+///     concat_as_sql_options(options)
+/// );
+/// ctx.sql(create_table_sql.as_str()).await?; // Query against this table
+///
+/// ```
+#[derive(Debug)]
 pub struct HudiTableFactory {}
 
 impl HudiTableFactory {
@@ -205,20 +330,21 @@ mod tests {
     use super::*;
     use datafusion::execution::session_state::SessionStateBuilder;
     use datafusion::prelude::{SessionConfig, SessionContext};
-    use datafusion_common::{DataFusionError, ScalarValue};
+    use datafusion_common::{Column, DataFusionError, ScalarValue};
     use std::fs::canonicalize;
     use std::path::Path;
     use std::sync::Arc;
     use url::Url;
 
+    use datafusion::logical_expr::BinaryExpr;
     use hudi_core::config::read::HudiReadConfig::InputPartitions;
-    use hudi_tests::TestTable::{
+    use hudi_test::SampleTable::{
         V6ComplexkeygenHivestyle, V6Empty, V6Nonpartitioned, V6SimplekeygenHivestyleNoMetafields,
         V6SimplekeygenNonhivestyle, V6SimplekeygenNonhivestyleOverwritetable,
         V6TimebasedkeygenNonhivestyle,
     };
-    use hudi_tests::{utils, TestTable};
-    use utils::{get_bool_column, get_i32_column, get_str_column};
+    use hudi_test::{util, SampleTable};
+    use util::{get_bool_column, get_i32_column, get_str_column};
 
     use crate::HudiDataSource;
     use crate::HudiTableFactory;
@@ -235,7 +361,7 @@ mod tests {
     #[tokio::test]
     async fn test_get_empty_schema_from_empty_table() {
         let table_provider =
-            HudiDataSource::new_with_options(V6Empty.path().as_str(), empty_options())
+            HudiDataSource::new_with_options(V6Empty.path_to_cow().as_str(), empty_options())
                 .await
                 .unwrap();
         let schema = table_provider.schema();
@@ -243,7 +369,7 @@ mod tests {
     }
 
     async fn register_test_table_with_session<I, K, V>(
-        test_table: &TestTable,
+        test_table: &SampleTable,
         options: I,
         use_sql: bool,
     ) -> Result<SessionContext, DataFusionError>
@@ -257,12 +383,12 @@ mod tests {
             let create_table_sql = format!(
                 "CREATE EXTERNAL TABLE {} STORED AS HUDI LOCATION '{}' {}",
                 test_table.as_ref(),
-                test_table.path(),
+                test_table.path_to_cow(),
                 concat_as_sql_options(options)
             );
             ctx.sql(create_table_sql.as_str()).await?;
         } else {
-            let base_url = test_table.url();
+            let base_url = test_table.url_to_cow();
             let hudi = HudiDataSource::new_with_options(base_url.as_str(), options).await?;
             ctx.register_table(test_table.as_ref(), Arc::new(hudi))?;
         }
@@ -311,7 +437,7 @@ mod tests {
             "CREATE EXTERNAL TABLE {} STORED AS {} LOCATION '{}'",
             test_table.as_ref(),
             invalid_format,
-            test_table.path()
+            test_table.path_to_cow()
         );
 
         let ctx = create_test_session().await;
@@ -419,4 +545,59 @@ mod tests {
             verify_data_with_replacecommits(&ctx, &sql, test_table.as_ref()).await
         }
     }
+
+    #[tokio::test]
+    async fn test_supports_filters_pushdown() {
+        let table_provider = HudiDataSource::new_with_options(
+            V6Nonpartitioned.path_to_cow().as_str(),
+            empty_options(),
+        )
+        .await
+        .unwrap();
+
+        let expr0 = Expr::BinaryExpr(BinaryExpr {
+            left: Box::new(Expr::Column(Column::from_name("name".to_string()))),
+            op: Operator::Eq,
+            right: Box::new(Expr::Literal(ScalarValue::Utf8(Some("Alice".to_string())))),
+        });
+
+        let expr1 = Expr::BinaryExpr(BinaryExpr {
+            left: Box::new(Expr::Column(Column::from_name("intField".to_string()))),
+            op: Operator::Gt,
+            right: Box::new(Expr::Literal(ScalarValue::Int32(Some(20000)))),
+        });
+
+        let expr2 = Expr::BinaryExpr(BinaryExpr {
+            left: Box::new(Expr::Column(Column::from_name(
+                "nonexistent_column".to_string(),
+            ))),
+            op: Operator::Eq,
+            right: Box::new(Expr::Literal(ScalarValue::Int32(Some(1)))),
+        });
+
+        let expr3 = Expr::BinaryExpr(BinaryExpr {
+            left: Box::new(Expr::Column(Column::from_name("name".to_string()))),
+            op: Operator::NotEq,
+            right: Box::new(Expr::Literal(ScalarValue::Utf8(Some("Diana".to_string())))),
+        });
+
+        let expr4 = Expr::Literal(ScalarValue::Int32(Some(10)));
+
+        let expr5 = Expr::Not(Box::new(Expr::BinaryExpr(BinaryExpr {
+            left: Box::new(Expr::Column(Column::from_name("intField".to_string()))),
+            op: Operator::Gt,
+            right: Box::new(Expr::Literal(ScalarValue::Int32(Some(20000)))),
+        })));
+
+        let filters = vec![&expr0, &expr1, &expr2, &expr3, &expr4, &expr5];
+        let result = table_provider.supports_filters_pushdown(&filters).unwrap();
+
+        assert_eq!(result.len(), 6);
+        assert_eq!(result[0], TableProviderFilterPushDown::Inexact);
+        assert_eq!(result[1], TableProviderFilterPushDown::Inexact);
+        assert_eq!(result[2], TableProviderFilterPushDown::Unsupported);
+        assert_eq!(result[3], TableProviderFilterPushDown::Inexact);
+        assert_eq!(result[4], TableProviderFilterPushDown::Unsupported);
+        assert_eq!(result[5], TableProviderFilterPushDown::Inexact);
+    }
 }
