@@ -15,6 +15,14 @@
 #  specific language governing permissions and limitations
 #  under the License.
 
-from ._internal import __version__ as __version__
-from ._internal import HudiFileSlice as HudiFileSlice
-from ._internal import HudiTable as HudiTable
+
+from hudi._internal import HudiFileGroupReader, HudiFileSlice, HudiTable
+from hudi._internal import __version__ as __version__
+from hudi.table.builder import HudiTableBuilder
+
+__all__ = [
+    "HudiFileGroupReader",
+    "HudiFileSlice",
+    "HudiTable",
+    "HudiTableBuilder",
+]
